@@ -3,26 +3,26 @@
 import os
 import sys
 import tempfile
-from typing import List
 
 import auto_editor
 import auto_editor.utils.func as usefulfunctions
 from auto_editor.edit import edit_media
 from auto_editor.ffwrapper import FFmpeg
 from auto_editor.utils.log import Log, Timer
+from auto_editor.utils.types import MainArgs
 from auto_editor.validate_input import valid_input
 from auto_editor.vanparse import ArgumentParser
 
 
 def main_options(parser: ArgumentParser) -> ArgumentParser:
     from auto_editor.utils.types import (
-        color_type,
-        float_type,
-        frame_type,
-        margin_type,
-        range_type,
-        sample_rate_type,
-        speed_range_type,
+        color,
+        margin,
+        number,
+        sample_rate,
+        speed_range,
+        time,
+        time_range,
     )
 
     parser.add_text("Object Options")
@@ -47,9 +47,7 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
         help="Add an image object onto the timeline.",
     )
     parser.add_text("URL Download Options")
-    parser.add_argument(
-        "--yt-dlp-location", default="yt-dlp", help="Set a custom path to yt-dlp."
-    )
+    parser.add_argument("--yt-dlp-location", help="Set a custom path to yt-dlp.")
     parser.add_argument(
         "--download-format", help="Set the yt-dlp download format. (--format, -f)"
     )
@@ -64,39 +62,33 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
         "--video-codec",
         "-vcodec",
         "-c:v",
-        default="auto",
         help="Set the video codec for the output media file.",
     )
     parser.add_argument(
         "--audio-codec",
         "-acodec",
         "-c:a",
-        default="auto",
         help="Set the audio codec for the output media file.",
     )
     parser.add_argument(
         "--video-bitrate",
         "-b:v",
-        default="10m",
         help="Set the number of bits per second for video.",
     )
     parser.add_argument(
         "--audio-bitrate",
         "-b:a",
-        default="unset",
         help="Set the number of bits per second for audio.",
     )
     parser.add_argument(
         "--video-quality-scale",
         "-qscale:v",
         "-q:v",
-        default="unset",
         help="Set a value to the ffmpeg option -qscale:v",
     )
     parser.add_argument(
         "--scale",
-        type=float_type,
-        default=1,
+        type=number,
         help="Scale the input video's resolution by the given factor.",
     )
     parser.add_argument(
@@ -111,34 +103,34 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
     parser.add_text("Manual Editing Options")
     parser.add_argument(
         "--cut-out",
-        type=range_type,
+        type=time_range,
         nargs="*",
         help="The range of media that will be removed completely, regardless of the "
         "value of silent speed.",
     )
     parser.add_argument(
         "--add-in",
-        type=range_type,
+        type=time_range,
         nargs="*",
         help="The range of media that will be added in, opposite of --cut-out",
     )
     parser.add_blank()
     parser.add_argument(
         "--mark-as-loud",
-        type=range_type,
+        type=time_range,
         nargs="*",
         help='The range that will be marked as "loud".',
     )
     parser.add_argument(
         "--mark-as-silent",
-        type=range_type,
+        type=time_range,
         nargs="*",
         help='The range that will be marked as "silent".',
     )
     parser.add_argument(
         "--set-speed-for-range",
         "--set-speed",
-        type=speed_range_type,
+        type=speed_range,
         nargs="*",
         help="SPEED,START,STOP - Set an arbitrary speed for a given range.",
     )
@@ -153,31 +145,28 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
     parser.add_argument(
         "--sample-rate",
         "-ar",
-        type=sample_rate_type,
+        type=sample_rate,
         help="Set the sample rate for the timeline and output media.",
     )
     parser.add_argument(
         "--background",
-        type=color_type,
-        default="#000",
+        type=color,
         help="Set the color of the background that is visible when the video is moved.",
     )
     parser.add_text("Select Editing Source Options")
     parser.add_argument(
         "--edit-based-on",
         "--edit",
-        default="audio",
         help="Decide which method to use when making edits.",
     )
     parser.add_argument(
-        "--keep-tracks-seperate",
+        "--keep-tracks-separate",
         flag=True,
         help="Don't combine audio tracks when exporting.",
     )
     parser.add_argument(
         "--export",
         "-ex",
-        default="default",
         choices=[
             "default",
             "premiere",
@@ -209,7 +198,6 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
     parser.add_text("Display Options")
     parser.add_argument(
         "--progress",
-        default="modern",
         choices=["modern", "classic", "ascii", "machine", "none"],
         help="Set what type of progress bar to use.",
     )
@@ -229,56 +217,46 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
     parser.add_argument(
         "--timeline", flag=True, help="Show auto-editor JSON timeline file and halt."
     )
-    parser.add_argument(
-        "--api",
-        default="1.0.0",
-        help="Set what version of the JSON timeline to output.",
-    )
+    parser.add_argument("--api", help="Set what version of the timeline to use.")
     parser.add_text("Global Editing Options")
     parser.add_argument(
         "--silent-threshold",
         "-t",
-        type=float_type,
-        default=0.04,
+        type=number,
         help="Set the volume that frames audio needs to surpass to be marked loud.",
     )
     parser.add_argument(
         "--frame-margin",
         "--margin",
         "-m",
-        type=margin_type,
-        default="6",
+        type=margin,
         help='Set how many "silent" frames on either side of the "loud" sections to include.',
     )
     parser.add_argument(
         "--silent-speed",
         "-s",
-        type=float_type,
-        default=99999,
+        type=number,
         help='Set the speed that "silent" sections should be played at.',
     )
     parser.add_argument(
         "--video-speed",
         "--sounded-speed",
         "-v",
-        type=float_type,
-        default=1,
+        type=number,
         help='Set the speed that "loud" sections should be played at.',
     )
     parser.add_argument(
         "--min-clip-length",
         "-minclip",
         "-mclip",
-        type=frame_type,
-        default=3,
+        type=time,
         help="Set the minimum length a clip can be. If a clip is too short, cut it.",
     )
     parser.add_argument(
         "--min-cut-length",
         "-mincut",
         "-mcut",
-        type=frame_type,
-        default=6,
+        type=time,
         help="Set the minimum length a cut can be. If a cut is too short, don't cut.",
     )
     parser.add_blank()
@@ -296,7 +274,6 @@ def main_options(parser: ArgumentParser) -> ArgumentParser:
 
 
 def main() -> None:
-    parser = ArgumentParser("Auto-Editor")
     subcommands = ("test", "info", "levels", "grep", "subdump", "desc")
 
     if len(sys.argv) > 1 and sys.argv[1] in subcommands:
@@ -306,45 +283,19 @@ def main() -> None:
         obj.main(sys.argv[2:])
         sys.exit()
     else:
-        parser = main_options(parser)
-
-        # Preserve backwards compatibility
-
-        sys_a = sys.argv[1:]
-
-        def macro(sys_a: List[str], options: List[str], new: List[str]) -> List[str]:
-            for option in options:
-                if option in sys_a:
-                    pos = sys_a.index(option)
-                    sys_a[pos : pos + 1] = new
-            return sys_a
-
-        sys_a = macro(
-            sys_a,
-            ["--export_to_premiere", "--export-to-premiere", "-exp"],
-            ["--export", "premiere"],
+        args = main_options(ArgumentParser("Auto-Editor")).parse_args(
+            MainArgs,
+            sys.argv[1:],
+            macros=[
+                ({"--export-to-premiere", "-exp"}, ["--export", "premiere"]),
+                ({"--export-to-final-cut-pro", "-exf"}, ["--export", "final-cut-pro"]),
+                ({"--export-to-shotcut", "-exs"}, ["--export", "shotcut"]),
+                ({"--export-as-json"}, ["--export", "json"]),
+                ({"--export-as-clip-sequence", "-excs"}, ["--export", "clip-sequence"]),
+                ({"--combine-files"}, []),
+                ({"--keep-tracks-seperate"}, ["--keep-tracks-separate"]),
+            ],
         )
-        sys_a = macro(
-            sys_a,
-            ["--export_to_final_cut_pro", "--export-to-final-cut-pro", "-exf"],
-            ["--export", "final-cut-pro"],
-        )
-        sys_a = macro(
-            sys_a,
-            ["--export_to_shotcut", "--export-to-shotcut", "-exs"],
-            ["--export", "shotcut"],
-        )
-        sys_a = macro(
-            sys_a, ["--export_as_json", "--export-as-json"], ["--export", "json"]
-        )
-        sys_a = macro(
-            sys_a,
-            ["--export_as_clip_sequence", "--export-as-clip-sequence", "-excs"],
-            ["--export", "clip-sequence"],
-        )
-        sys_a = macro(sys_a, ["--combine-files", "--combine_files"], [])
-
-        args = parser.parse_args(sys_a)
 
     timer = Timer(args.quiet)
 
