@@ -1,12 +1,12 @@
 import os
-from typing import List, Optional, Tuple
+from typing import List, Optional, Type
 
 from auto_editor.ffwrapper import FFmpeg, FileInfo
 from auto_editor.timeline import Timeline, make_timeline
 from auto_editor.utils.container import get_rules
 from auto_editor.utils.log import Log
 from auto_editor.utils.progressbar import ProgressBar
-from auto_editor.utils.types import ChunkType
+from auto_editor.utils.types import Chunk, Chunks, MainArgs
 
 
 def set_output_name(path: str, inp_ext: str, export: str) -> str:
@@ -29,7 +29,7 @@ def set_output_name(path: str, inp_ext: str, export: str) -> str:
 
 
 def edit_media(
-    path_list: List[str], ffmpeg: FFmpeg, args, temp: str, log: Log
+    path_list: List[str], ffmpeg: FFmpeg, args: Type[MainArgs], temp: str, log: Log
 ) -> Optional[str]:
 
     progress = ProgressBar(args.progress)
@@ -97,7 +97,7 @@ def edit_media(
         if acodec in rules.disallow_a:
             log.error(codec_error.format(acodec, output_container))
 
-    if args.keep_tracks_seperate and rules.max_audios == 1:
+    if args.keep_tracks_separate and rules.max_audios == 1:
         log.warning(
             f"'{output_container}' container doesn't support multiple audio tracks."
         )
@@ -161,7 +161,7 @@ def edit_media(
     if args.preview:
         from auto_editor.preview import preview
 
-        preview(timeline, log)
+        preview(timeline, temp, log)
         return None
 
     if args.export == "json":
@@ -235,9 +235,11 @@ def edit_media(
         from auto_editor.timeline import clipify, make_av
         from auto_editor.utils.func import append_filename
 
-        def pad_chunk(chunk: Tuple[int, int, float], total_frames: int) -> ChunkType:
-            start = [] if chunk[0] == 0 else [(0, chunk[0], 99999)]
-            end = [] if chunk[1] == total_frames else [(chunk[1], total_frames, 99999)]
+        def pad_chunk(chunk: Chunk, total_frames: int) -> Chunks:
+            start = [] if chunk[0] == 0 else [(0, chunk[0], 99999.0)]
+            end = (
+                [] if chunk[1] == total_frames else [(chunk[1], total_frames, 99999.0)]
+            )
             return start + [chunk] + end
 
         clip_num = 0
