@@ -9,7 +9,7 @@ from re import search
 from subprocess import PIPE, Popen
 from typing import List, Optional, Tuple
 
-from auto_editor.utils.func import get_stdout, to_timecode
+from auto_editor.utils.func import get_stdout
 from auto_editor.utils.log import Log
 
 IMG_CODECS = ("png", "mjpeg", "webp")
@@ -25,9 +25,17 @@ class FFmpeg:
             return ff_location
         if my_ffmpeg or system() not in ("Windows", "Darwin"):
             return "ffmpeg"
-        program = "ffmpeg" if system() == "Darwin" else "ffmpeg.exe"
+
+        # Assumming accessing __file__ is somewhat expensive.
+        program = "ffmpeg.exe" if system() == "Windows" else "ffmpeg"
         dirpath = os.path.dirname(os.path.realpath(__file__))
-        return os.path.join(dirpath, "ffmpeg", system(), program)
+        file_path = os.path.join(dirpath, "ffmpeg", system(), program)
+
+        # May not exist because auto-editor doesn't bundle every architecture.
+        if os.path.isfile(file_path):
+            return file_path
+
+        return "ffmpeg"
 
     def __init__(
         self,
@@ -167,8 +175,6 @@ class FileInfo:
         "dirname",
         "name",
         "ext",
-        "duration",
-        "fdur",
         "bitrate",
         "description",
         "videos",
@@ -232,13 +238,14 @@ class FileInfo:
             if "format" not in json_info:
                 raise ValueError("Key 'format' not found")
         except Exception as e:
-            log.error(f"Could not read ffprobe JSON: {e}")
+            log.error(f"{path}: Could not read ffprobe JSON: {e}")
 
-        self.fdur = float(json_info["format"]["duration"])
-        self.duration = to_timecode(self.fdur, "standard")
         self.bitrate = json_info["format"]["bit_rate"]
-        if "description" in json_info["format"]:
-            self.description = json_info["format"]["description"]
+        if (
+            "tags" in json_info["format"]
+            and "description" in json_info["format"]["tags"]
+        ):
+            self.description = json_info["format"]["tags"]["description"]
 
         for stream in json_info["streams"]:
             lang = None
@@ -265,7 +272,7 @@ class FileInfo:
             if codec_type == "video":
                 codec = stream["codec_name"]
                 try:
-                    fps = Fraction(stream["avg_frame_rate"])
+                    fps = float(Fraction(stream["avg_frame_rate"]))
                 except ZeroDivisionError:
                     fps = 0
                 except ValueError:
