@@ -5,15 +5,17 @@ Make a pre-edited file reference that can be inputted back into auto-editor.
 import json
 import os
 import sys
-from typing import Any, Union
+from typing import Any, Optional, Union
 
 from auto_editor.ffwrapper import FFmpeg, FileInfo
 from auto_editor.timeline import Timeline, clipify, make_av
 from auto_editor.utils.log import Log
-from auto_editor.utils.types import ChunkType
+from auto_editor.utils.types import Chunks
 
 
 def check_attrs(data: object, log: Log, *attrs: str) -> None:
+    if not isinstance(data, dict):
+        log.error("Data is in wrong shape!")
     for attr in attrs:
         if attr not in data:
             log.error(f"'{attr}' attribute not found!")
@@ -24,7 +26,7 @@ def check_file(path: str, log: Log):
         log.error(f"Could not locate media file: '{path}'")
 
 
-def validate_chunks(chunks: object, log: Log) -> ChunkType:
+def validate_chunks(chunks: object, log: Log) -> Chunks:
     if not isinstance(chunks, (list, tuple)):
         log.error("Chunks must be a list")
 
@@ -32,11 +34,9 @@ def validate_chunks(chunks: object, log: Log) -> ChunkType:
         log.error("Chunks are empty!")
 
     new_chunks = []
-
-    prev_end = None
+    prev_end: Optional[int] = None
 
     for i, chunk in enumerate(chunks):
-
         if len(chunk) != 3:
             log.error("Chunk must have a length of 3.")
 
