@@ -6,6 +6,7 @@ import platform
 import shutil
 import subprocess
 import sys
+from dataclasses import dataclass, field
 from time import perf_counter
 from typing import Callable, List, NoReturn, Optional, Tuple
 
@@ -17,14 +18,15 @@ from auto_editor.vanparse import ArgumentParser
 av.logging.set_level(av.logging.PANIC)
 
 
+@dataclass
+class TestArgs:
+    only: List[str] = field(default_factory=list)
+    help: bool = False
+    category: str = "cli"
+
+
 def test_options(parser):
     parser.add_argument("--only", "-n", nargs="*")
-    parser.add_argument(
-        "--help",
-        "-h",
-        flag=True,
-        help="Print info about the program or an option and exit.",
-    )
     parser.add_required(
         "category",
         nargs=1,
@@ -122,7 +124,7 @@ def make_np_list(in_file: str, compare_file: str, the_speed: float) -> None:
 
 
 class Tester:
-    def __init__(self, args):
+    def __init__(self, args: TestArgs) -> None:
         self.passed_tests = 0
         self.failed_tests = 0
         self.args = args
@@ -159,12 +161,10 @@ class Tester:
 
 
 def main(sys_args: Optional[List[str]] = None):
-    parser = test_options(ArgumentParser("test"))
-
     if sys_args is None:
         sys_args = sys.argv[1:]
 
-    args = parser.parse_args(sys_args)
+    args = test_options(ArgumentParser("test")).parse_args(TestArgs, sys_args)
 
     ### Tests ###
 
@@ -242,7 +242,6 @@ def main(sys_args: Optional[List[str]] = None):
             assert video.width == 1280
             assert video.height == 720
             assert video.codec.name == "mpeg4"
-            assert float(video.duration * video.time_base) == 17.633333333333333
             assert cn.streams.audio[0].codec.name == "aac"
             assert cn.streams.audio[0].rate == 48000
 
@@ -253,7 +252,6 @@ def main(sys_args: Optional[List[str]] = None):
             assert video.width == 1280
             assert video.height == 720
             assert video.codec.name == "h264"
-            assert float(video.duration * video.time_base) == 17.633333333333333
             assert cn.streams.audio[0].codec.name == "aac"
             assert cn.streams.audio[0].rate == 48000
             assert video.language == "eng"
@@ -403,11 +401,11 @@ def main(sys_args: Optional[List[str]] = None):
         os.remove("resources/testsrc_ALTERED.mp4")
 
     def render_text():
-        run_program(["example.mp4", "--add-text", "start,end,This is my text"])
+        run_program(["example.mp4", "--add-text", "0,30,This is my text,font=default"])
 
     def check_font_error():
         check_for_error(
-            ["example.mp4", "--add-text", "start,end,text,0,0,30,notafont"], "not found"
+            ["example.mp4", "--add-text", "0,30,text,0,0,30,notafont"], "not found"
         )
 
     def export_tests():
@@ -446,21 +444,21 @@ def main(sys_args: Optional[List[str]] = None):
             video = cn.streams.video[0]
             assert video.average_rate == 15
             dur = float(video.duration * video.time_base)
-            assert dur - 17.633333333333333 < 3
+            assert dur - 17.33333333333333333333333 < 3
 
         run_program(["example.mp4", "-r", "20"])
         with av.open("example_ALTERED.mp4", "r") as cn:
             video = cn.streams.video[0]
             assert video.average_rate == 20
             dur = float(video.duration * video.time_base)
-            assert dur - 17.633333333333333 < 2
+            assert dur - 17.33333333333333333333333 < 2
 
         run_program(["example.mp4", "-r", "60"])
         with av.open("example_ALTERED.mp4", "r") as cn:
             video = cn.streams.video[0]
             assert video.average_rate == 60
             dur = float(video.duration * video.time_base)
-            assert dur - 17.633333333333333 < 0.3
+            assert dur - 17.33333333333333333333333 < 0.3
 
     def image_test():
         run_program(["resources/embedded-image/h264-png.mp4"])
@@ -576,6 +574,7 @@ def main(sys_args: Optional[List[str]] = None):
                 help_tests,
                 version_test,
                 parser_test,
+                combine,
                 example_tests,
                 export_tests,
                 high_speed_test,
